--------------------------------------------------------------------------------------------
-- Broker_TSMProfile - Displays the current TSM profile and lets you switch between profiles
--------------------------------------------------------------------------------------------
-- Broker_TSMProfile.lua
--
--[[

Have you ever placed a lot of items to the auction house using a wrong Trade Skill Master profile?

Broker_TSMProfile displays the current Trade Skill Master profile and lets you
switch to a different profile with a simple click, so you will never lose gold again
because the wrong profile was set.

Using a Data Broker Display is recommended but is not necessary.
Broker_TSMProfile has LDB support for Bazooka, ChocolateBar, Titan Panel, StatBlockCore etc.

Author: Caraxe
All Rights Reserved

Version 0.1.4
------------------------------------------------------------------------------------]]

-----------------------------------------------------------------------------
-- Initializing Addon

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local addonVersionString = GetAddOnMetadata(addonName, "Version")

local AceAddon = LibStub("AceAddon-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local LibQTip = LibStub("LibQTip-1.0")
local LDB = LibStub("LibDataBroker-1.1")
local Icon = LibStub("LibDBIcon-1.0")

-----------------------------------------------------------------------------
-- Encounter TSM object reference hiding

local TSMAddonName = ""
local TSMAddonObject = nil

local function NewAddonH(objectorname, ...)
	local obj = NewAddonO(objectorname, ...)
	if obj then
		if obj.name == "TradeSkillMaster" then
			TSMAddonName = obj.name
			TSMAddonObject = obj
			-- found it, back to normal
			AceAddon.NewAddon = NewAddonO
		end
	end
	return obj
end

if AceAddon then
	NewAddonO = AceAddon.NewAddon
	AceAddon.NewAddon = NewAddonH
end

-----------------------------------------------------------------------------
-- LDB Integration

local function do_OnEnter(frame)
	local tooltip = LibQTip:Acquire(addon)
	tooltip:SmartAnchorTo(frame)
	tooltip:SetAutoHideDelay(0.1, frame)
	tooltip:EnableMouse(true)
	addon:UpdateTooltip()
	tooltip:Show()
end

local function do_OnLeave()
	-- empty dummy
end

local function do_OnClick(frame, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
	end
	if button == "LeftButton" then
		local editbox = ChatEdit_ChooseBoxForSend(DEFAULT_CHAT_FRAME)
		ChatEdit_ActivateChat(editbox)
		editbox:SetText("/tsm")
		ChatEdit_OnEnterPressed(editbox)
	end
	addon:UpdateTooltip()
end

addon.LDB = LDB:NewDataObject(addonName, {
	type = "data source",
	label = addonName,
	text = L["TSM Profile"] .. ": --",
	icon = "Interface\\Icons\\Trade_Engineering",
	OnEnter = do_OnEnter,
	OnLeave = do_OnLeave,
	OnClick = do_OnClick,
});

-----------------------------------------------------------------------------
-- Options

local defaults = {
	profile = {
		minimap = {
			hide = false,
		},
	},
	global = {},
}

local function GetOptions(uiType, uiName, appName)
	if appName == addonName then
		local options = {
				type = "group",
				name = GetAddOnMetadata(addonName, "Title"),
				args = {
					brfudesc = {
						type = "description",
						order = 0,
						name = GetAddOnMetadata(addonName, "Notes"),
					},
					minimap = {
						name = L["Minimap Icon"],
						desc = L["Toggle minimap icon"],
						type = "toggle",
						order = 10,
						get = function()
							local db = addon.db.profile
							return not db.minimap.hide
							end,
						set = function()
							local db = addon.db.profile
							db.minimap.hide = not db.minimap.hide
							if db.minimap.hide then
								Icon:Hide(addonName)
							else
								Icon:Show(addonName)
							end
						end,
					},
				},
		}
		return options
	end
end

-----------------------------------------------------------------------------
-- Broker_TSMProfile

local function SecTimer()
	local profile = ""
	if TSMAddonObject then
		profile = TSMAddonObject.db:GetCurrentProfile()
	end
	addon.LDB.text = L["TSM Profile"] .. ": " .. profile
end

function addon:OnInitialize()
	addon.db = AceDB:New("Broker_TSMProfileDB", defaults, true)

	local db = addon.db.profile
	Icon:Register(addonName, addon.LDB, db.minimap)

	addon.externalsInfo = ""
	local TSMAddonVersion = GetAddOnMetadata("TradeSkillMaster", "Version")
	if TSMAddonVersion then
		addon.externalsInfo = "Trade Skill Master " .. TSMAddonVersion
	end

	AceConfigRegistry:RegisterOptionsTable(addonName, GetOptions)
	local optionsFrame = AceConfigDialog:AddToBlizOptions(addonName, GetAddOnMetadata(addonName, "Title"))
end

function addon:OnEnable()
	if TSMAddonObject then
		-- for detecting switches
		addon.sectimer = C_Timer.NewTicker(1, SecTimer)

		local profile = TSMAddonObject.db:GetCurrentProfile()
		addon:Printf(addonVersionString .. ", " .. addon.externalsInfo .. L[" detected"] .. ", " .. L["TSM Profile"] .. ": " .. profile)

		-- leave it hidden
		-- AceAddon.addons[TSMAddonName] = TSMAddonObject
	else
		addon:Printf(addonVersionString .. ", " .. L["no TSM object found"] .. ".")
	end
end

local function tooltip_line_OnMouseUp(frame, profile, button)
	if TSMAddonObject then
		TSMAddonObject:ChangeProfile(profile)
		addon.LDB.text = L["TSM Profile"] .. ": " .. profile
	end
	addon:UpdateTooltip()
end

function addon:UpdateTooltip()
	if not LibQTip:IsAcquired(addon) then
		return
	end

	local tooltip = LibQTip:Acquire(addon)
	tooltip:Clear()
	tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

	local line = tooltip:AddHeader()
	tooltip:SetCell(line, 1, "|cfffed100Broker_|cffffffffTSMProfile", nil, "CENTER", 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, L["Profile"])
	tooltip:SetCell(line, 2, L["State"])
	line = tooltip:AddSeparator()

	if TSMAddonObject then
		for i, profile in ipairs(TSMAddonObject.db:GetProfiles()) do
			line = tooltip:AddLine()
			tooltip:SetCell(line, 1, format("%d. %s", i, profile))
			if (TSMAddonObject.db:GetCurrentProfile() == profile) then
				tooltip:SetCell(line, 2, L["On"])
			else
				tooltip:SetCell(line, 2, "--")
			end
			tooltip:SetLineScript(line, "OnMouseUp", tooltip_line_OnMouseUp, profile)
		end
	end

	line = tooltip:AddSeparator()
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cfffed100" .. L["Click on profile to switch"], nil, nil, 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cfffed100" .. L["Right-Click opens options"], nil, nil, 2)

	tooltip:UpdateScrolling()
end

-- EOF
